package java.de.TimeSlice;

import java.util.*;
import java.security.*;

/**
 * @author      Burghard Grueter
 * @author      Johannes Schick
 * @version     1.0, 11/06/2000
 * @since       JDK1.2
 */

	/**
	 * class <pre>TaskManager</pre> is one of the
	 * main processing classes. It manages the
	 * existing tasks and the active task. It is
	 * called to update periodically by the Timer
	 * by implementing <pre>TimerListener</pre>
	 * Changes due to user-interaction with the GUI
	 * are notified trough <pre>ControlPanelListener</pre>
	 */

public class TaskManager implements ControlPanelListener, TimerListener
{
	
	public final static long MAXPERIODE=  Long.parseLong("31536000000");
	
	private Hashtable Tasks;
	private Task ActiveTask;
	

	/**
	 * some constructors are really boring
	 *
	 */
	public TaskManager()
	{
		Tasks = new Hashtable();
		ActiveTask = null;
	}


	/**
	 * add Task t to <pre>Tasks</pre> if it fulfills all 
	 * restrictions like uniqueness of the Taskname and
	 * ranges of data
	 *
	 * @param   t   Task to add
	 * @exception   TaskNameAlreadyExist  
	 * @exception   InvalidParameterException  
	 */
	public void addTask (Task t) throws TaskNameAlreadyExist, InvalidParameterException
	{
		
	    if (!Tasks.isEmpty() && Tasks.get(t.getTaskName())!= null)
			throw new TaskNameAlreadyExist(t.getTaskName()); 	    
		
		if (!checkBounds(t.getPeriode(),0, MAXPERIODE))
			throw new InvalidParameterException("Wert fr Periode ungltig");	
				
		if (!checkBounds(t.getDauer(), 0, t.getPeriode())) 
			throw new InvalidParameterException("Wert fr Dauer ungltig");
			
		Tasks.put(t.getTaskName(),t);
	   
	}
	

	/**
	 * nearly the same like <pre>addTask()</pre> but the existing 
	 * Task is preserved and the properties are set to the new values
	 *
	 * @param   OldName  
	 * @param   t  
	 * @exception   TaskNameAlreadyExist  
	 * @exception   InvalidParameterException  
	 */
	public void editTask(String OldName, Task t) throws TaskNameAlreadyExist, InvalidParameterException
	{
		 
     	if (!OldName.equals(t.getTaskName()) && (Tasks.get(t.getTaskName())!=null))
			throw new TaskNameAlreadyExist(t.getTaskName());
			
			
		if (!checkBounds(t.getPeriode(),0, MAXPERIODE))
			throw new InvalidParameterException("Wert fr Periode ungltig");	
					
		if (!checkBounds(t.getDauer(), 0, t.getPeriode())) 
			throw new InvalidParameterException("Wert fr Dauer ungltig");
					
		Task h = (Task) Tasks.get(OldName);
		h.assign(t.getTaskName(), t.getDauer(), t.getPeriode());
		Tasks.remove(OldName);
		Tasks.put(h.getTaskName(), h);		
	}
	
	

	/**
	 * since the timer accesses <pre>ActiveTask</pre> periodically
	 * one has to escape access conflicts by a synchronized access
	 * 
	 * @param   TaskName  
	 */
	public void deleteTask (String TaskName) 
	{   
		if (Tasks.get(TaskName)==ActiveTask)
	  	{
			synchronized(ActiveTask) 
			{
    			Tasks.remove(TaskName);
			}
		}
		else
		  Tasks.remove(TaskName);
	}

	/**
	 * sets the stae of the Task named <pre>TaskName</pre>
	 * to <pre>active</pre> and has to check and adjust
	 * <pre>ActiveTask</pre> 
	 *
	 * @param   active  
	 * @param   TaskName  
	 */
	public void activateTask (boolean active, String TaskName)
	{
		Task h = (Task) Tasks.get(TaskName);
	
		if (active) 
		{
			if (ActiveTask!=h)
			{
				if (ActiveTask!=null)
  				  ActiveTask.setActive(false);
				ActiveTask = h;
			}
		}
		else
		  if (ActiveTask==h) 
		    ActiveTask=null;
			 
		h.setActive(active);
		
	}

	private boolean checkBounds(long Value, long min, long max)
	{
		return ((Value>min) && (Value<max));
	};
	
	
	public Task getActiveTask () 
	{
	 	return ActiveTask;	
	}

	public void setActiveTask (Task Value) 
	{
		ActiveTask.setActive(false);
		ActiveTask = Value;
		ActiveTask.setActive(true);		
	}

	public Task getTask (String TaskName) 
	{
	 	return (Task) Tasks.get(TaskName);
	}

	public void setTask (String OldName, Task Value) 
	{
		Task h = (Task) this.Tasks.get(OldName);
		
		if (h!=null)
			h = Value;		
	}
	

	/**
	 * part of the interface <pre>TimerListener</pre>
	 * called periodically. Updates the <pre>Restdauer</pre>
	 * of the currently active Task (only one Task can be
	 * active a time). Like explained in the preceeding this
	 * access has to be synchronized
	 *
	 * @param   delta  
	 */
    public void updateActiveTask (long delta)
    {		
		if (ActiveTask!=null) 
		{
			synchronized(ActiveTask)
			{
	  		    ActiveTask.mindereRestdauer(delta);
			}
		}	
    }
	

	/**
	 * under construction !!
	 * 
	 * @param   now  
	 */
	public void updateDayChange (long now)
	{
		Enumeration t = Tasks.elements();
		
		while (t.hasMoreElements()) 
		{
		   Task h = (Task) t.nextElement();
			if ((now-h.getStart())>=h.getPeriode()) 
			{
				h.reset();
			}
		}
	}

}



 